declare type Susbcription = (entity: Entity, added: Component<any>[], removed: Component<any>[]) => void;
/**
 * Representation of an entity in ECS
 */
export declare abstract class Entity {
    /**
     * Lista de interessados sobre a atualiação dos componentes
     */
    private subscriptions;
    /**
     * Informs that subscriptions have been triggered.
     */
    private queued;
    /**
     * Quais componentes foram adicionados nessa entidade antes de informar aos interessados
     */
    private added;
    /**
     *  Quais componentes foram removidos dessa entidade antes de informar aos interessados
     */
    private removed;
    /**
     * Components by type
     */
    private components;
    id: number;
    /**
     * Informs if the entity is active
     */
    active: boolean;
    constructor();
    private dispatch;
    /**
     * Allows interested parties to receive information when this entity's component list is updated
     *
     * @param handler
     */
    subscribe(handler: Susbcription): () => Entity;
    /**
     * Add a component to this entity
     *
     * @param component
     */
    add(component: Component<any>): void;
    /**
     * Removes a component's reference from this entity
     *
     * @param component
     */
    remove(component: Component<any>): void;
}
/**
 * Force typing
 */
export declare type ComponentClassType<P> = (new (data: P) => Component<P>) & {
    /**
     * Static reference to type id
     */
    type: number;
    /**
     * Get all instances of this component from entity
     *
     * @param entity
     */
    allFrom(entity: Entity): Component<P>[];
    /**
     * Get one instance of this component from entity
     *
     * @param entity
     */
    oneFrom(entity: Entity): Component<P>;
};
/**
 * Representation of a component in ECS
 */
export declare abstract class Component<T> {
    /**
     * Register a new component class
     */
    static register<P>(): ComponentClassType<P>;
    type: number;
    data: T;
    constructor(type: number, data: T);
}
/**
 * Represents the logic that transforms component data of an entity from its current state to its next state. A system
 * runs on entities that have a specific set of component types.
 */
export declare abstract class System {
    /**
     * IDs of the types of components this system expects the entity to have before it can act on. If you want to
     * create a system that acts on all entities, enter [-1]
     */
    private components;
    /**
     * Unique identifier of an instance of this system
     */
    id: number;
    /**
     * The maximum times per second this system should be updated
     */
    frequence: number;
    /**
     * Invoked before updating entities available for this system. It is only invoked when there are entities with the
     * characteristics expected by this system.
     *
     * @param time
     */
    beforeUpdateAll?(time: number): void;
    /**
     * Invoked in updates, limited to the value set in the "frequency" attribute
     *
     * @param time
     * @param delta
     * @param entity
     */
    update?(time: number, delta: number, entity: Entity): void;
    /**
     * Invoked after performing update of entities available for this system. It is only invoked when there are entities
     * with the characteristics expected by this system.
     *
     * @param time
     */
    afterUpdateAll?(time: number, entities: Entity[]): void;
    /**
     * Invoked when an expected feature of this system is added or removed from the entity
     *
     * @param entity
     * @param added
     * @param removed
     */
    change?(entity: Entity, added: Component<any>[], removed: Component<any>[]): void;
    /**
     * Invoked when:
     * a) An entity with the characteristics (components) expected by this system is added in the world;
     * b) This system is added in the world and this world has one or more entities with the characteristics expected by
     * this system;
     * c) An existing entity in the same world receives a new component at runtime and all of its new components match
     * the standard expected by this system.
     *
     * @param entity
     */
    enter?(entity: Entity): void;
    /**
     * Invoked when:
     * a) An entity with the characteristics (components) expected by this system is removed from the world;
     * b) This system is removed from the world and this world has one or more entities with the characteristics
     * expected by this system;
     * c) An existing entity in the same world loses a component at runtime and its new component set no longer matches
     * the standard expected by this system
     *
     * @param entity
     */
    exit?(entity: Entity): void;
    /**
     * @param components IDs of the types of components this system expects the entity to have before it can act on.
     * If you want to create a system that acts on all entities, enter [-1]
     * @param frequence The maximum times per second this system should be updated. Defaults 0
     */
    constructor(components: number[], frequence?: number);
    getComponents(): number[];
}
/**
 * The very definition of the ECS. Also called Admin or Manager in other implementations.
 */
export default class ECS {
    static System: typeof System;
    static Entity: typeof Entity;
    static Component: typeof Component;
    /**
     * All systems in this world
     */
    private systems;
    /**
     * All entities in this world
     */
    private entities;
    /**
     * Indexes the systems that must be run for each entity
     */
    private entitySystems;
    /**
     * Records the last instant a system was run in this world for an entity
     */
    private entitySystemLastUpdate;
    /**
     * Saves subscriptions made to entities
     */
    private entitySubscription;
    constructor(systems?: System[]);
    /**
     * Get an entity by id
     *
     * @param id
     */
    getEntity(id: number): Entity | undefined;
    /**
     * Add an entity to this world
     *
     * @param entity
     */
    addEntity(entity: Entity): void;
    /**
     * Remove an entity from this world
     *
     * @param idOrInstance
     */
    removeEntity(idOrInstance: number | Entity): void;
    /**
     * Add a system in this world
     *
     * @param system
     */
    addSystem(system: System): void;
    /**
     * Remove a system from this world
     *
     * @param system
     */
    removeSystem(system: System): void;
    /**
     * Invokes the "update" method of the systems in this world.
     */
    update(): void;
    /**
     * When an entity receives or loses components, invoking the change method of the systems
     *
     * @param entity
     */
    private onEntityUpdate;
    private indexEntitySystem;
    /**
     * Indexes an entity
     *
     * @param entity
     */
    private indexEntity;
}
export {};
